@@ -2,6 +2,7 @@
  * ac3_parse.c: ac3 parsing procedures
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001 VideoLAN
+ * $Id: ac3_parse.c,v 1.19 2001/04/26 00:12:19 reno Exp $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
  *          Aaron Holtzman <aholtzma@engr.uvic.ca>
@@ -24,12 +25,22 @@
 
 #include "defs.h"
 
+#include "config.h"
 #include "common.h"
-#include "intf_msg.h"
 
+#include "threads.h"
+#include "mtime.h"
+
+#include "stream_control.h"
+#include "input_ext-dec.h"
+
+#include "audio_output.h"
+
+#include "intf_msg.h"
 #include "ac3_decoder.h"
+#include "ac3_decoder_thread.h"
+
 #include "ac3_internal.h"
-#include "ac3_bit_stream.h"
 
 /* Misc LUT */
 static const u16 nfchans[] = { 2, 1, 2, 3, 3, 4, 4, 5 };
@@ -90,31 +101,38 @@ void parse_audblk_stats (ac3dec_t * p_ac3dec);
 /* Parse a syncinfo structure */
 int ac3_sync_frame (ac3dec_t * p_ac3dec, ac3_sync_info_t * p_sync_info) 
 {
-    int buf;
+    ac3dec_thread_t * p_ac3dec_t = (ac3dec_thread_t *) p_ac3dec->bit_stream.p_callback_arg;
+    
+    p_ac3dec->total_bits_read = 0;
+    p_ac3dec->i_available = 0;
     
-    p_ac3dec->bit_stream.total_bits_read = 0;
-    p_ac3dec->bit_stream.i_available = 0;
-
     /* sync word - should be 0x0b77 */
-    buf =  bitstream_get(&(p_ac3dec->bit_stream),16);
-    if (buf != 0x0b77)
+    RealignBits(&p_ac3dec->bit_stream);
+    while( (ShowBits (&p_ac3dec->bit_stream,16)) != 0x0b77 && 
+            (!p_ac3dec_t->p_fifo->b_die) && (!p_ac3dec_t->p_fifo->b_error))
     {
-        return 1;
+        RemoveBits (&p_ac3dec->bit_stream,8);
+        p_ac3dec->total_bits_read += 8;
     }
+    RemoveBits (&p_ac3dec->bit_stream,16);
+    p_ac3dec->total_bits_read += 16;
 
+    
     /* Get crc1 - we don't actually use this data though */
-    bitstream_get(&(p_ac3dec->bit_stream),16);
+    GetBits (&p_ac3dec->bit_stream,16);
 
     /* Get the sampling rate */
-    p_ac3dec->syncinfo.fscod = bitstream_get(&(p_ac3dec->bit_stream),2);
+    p_ac3dec->syncinfo.fscod = GetBits (&p_ac3dec->bit_stream,2);
 
     if (p_ac3dec->syncinfo.fscod >= 3)
     {
+        p_ac3dec->total_bits_read += 34;
         return 1;
     }
 
     /* Get the frame size code */
-    p_ac3dec->syncinfo.frmsizecod = bitstream_get(&(p_ac3dec->bit_stream),6);
+    p_ac3dec->syncinfo.frmsizecod = GetBits (&p_ac3dec->bit_stream,6);
+    p_ac3dec->total_bits_read += 40;
 
     if (p_ac3dec->syncinfo.frmsizecod >= 38)
     {
@@ -137,7 +155,7 @@ int ac3_sync_frame (ac3dec_t * p_ac3dec, ac3_sync_info_t * p_sync_info)
 int parse_bsi (ac3dec_t * p_ac3dec)
 {
     /* Check the AC-3 version number */
-    p_ac3dec->bsi.bsid = bitstream_get(&(p_ac3dec->bit_stream),5);
+    p_ac3dec->bsi.bsid = GetBits (&p_ac3dec->bit_stream,5);
 
     if (p_ac3dec->bsi.bsid > 8)
     {
@@ -145,10 +163,11 @@ int parse_bsi (ac3dec_t * p_ac3dec)
     }
 
     /* Get the audio service provided by the stream */
-    p_ac3dec->bsi.bsmod = bitstream_get(&(p_ac3dec->bit_stream),3);
+    p_ac3dec->bsi.bsmod = GetBits (&p_ac3dec->bit_stream,3);
 
     /* Get the audio coding mode (ie how many channels)*/
-    p_ac3dec->bsi.acmod = bitstream_get(&(p_ac3dec->bit_stream),3);
+    p_ac3dec->bsi.acmod = GetBits (&p_ac3dec->bit_stream,3);
+    
     /* Predecode the number of full bandwidth channels as we use this
      * number a lot */
     p_ac3dec->bsi.nfchans = nfchans[p_ac3dec->bsi.acmod];
@@ -156,115 +175,129 @@ int parse_bsi (ac3dec_t * p_ac3dec)
     /* If it is in use, get the centre channel mix level */
     if ((p_ac3dec->bsi.acmod & 0x1) && (p_ac3dec->bsi.acmod != 0x1))
     {
-        p_ac3dec->bsi.cmixlev = bitstream_get(&(p_ac3dec->bit_stream),2);
+        p_ac3dec->bsi.cmixlev = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->total_bits_read += 2;
     }
 
     /* If it is in use, get the surround channel mix level */
     if (p_ac3dec->bsi.acmod & 0x4)
     {
-        p_ac3dec->bsi.surmixlev = bitstream_get(&(p_ac3dec->bit_stream),2);
+        p_ac3dec->bsi.surmixlev = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->total_bits_read += 2;
     }
 
     /* Get the dolby surround mode if in 2/0 mode */
     if (p_ac3dec->bsi.acmod == 0x2)
     {
-        p_ac3dec->bsi.dsurmod = bitstream_get(&(p_ac3dec->bit_stream),2);
+        p_ac3dec->bsi.dsurmod = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->total_bits_read += 2;
     }
 
     /* Is the low frequency effects channel on? */
-    p_ac3dec->bsi.lfeon = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->bsi.lfeon = GetBits (&p_ac3dec->bit_stream,1);
 
     /* Get the dialogue normalization level */
-    p_ac3dec->bsi.dialnorm = bitstream_get(&(p_ac3dec->bit_stream),5);
+    p_ac3dec->bsi.dialnorm = GetBits (&p_ac3dec->bit_stream,5);
 
     /* Does compression gain exist? */
-    if ((p_ac3dec->bsi.compre = bitstream_get(&(p_ac3dec->bit_stream),1)))
+    if ((p_ac3dec->bsi.compre = GetBits (&p_ac3dec->bit_stream,1)))
     {
         /* Get compression gain */
-        p_ac3dec->bsi.compr = bitstream_get(&(p_ac3dec->bit_stream),8);
+        p_ac3dec->bsi.compr = GetBits (&p_ac3dec->bit_stream,8);
+        p_ac3dec->total_bits_read += 8;
     }
 
     /* Does language code exist? */
-    if ((p_ac3dec->bsi.langcode = bitstream_get(&(p_ac3dec->bit_stream),1)))
+    if ((p_ac3dec->bsi.langcode = GetBits (&p_ac3dec->bit_stream,1)))
     {
         /* Get langauge code */
-        p_ac3dec->bsi.langcod = bitstream_get(&(p_ac3dec->bit_stream),8);
+        p_ac3dec->bsi.langcod = GetBits (&p_ac3dec->bit_stream,8);
+        p_ac3dec->total_bits_read += 8;
     }
 
     /* Does audio production info exist? */
-    if ((p_ac3dec->bsi.audprodie = bitstream_get(&(p_ac3dec->bit_stream),1)))
+    if ((p_ac3dec->bsi.audprodie = GetBits (&p_ac3dec->bit_stream,1)))
     {
         /* Get mix level */
-        p_ac3dec->bsi.mixlevel = bitstream_get(&(p_ac3dec->bit_stream),5);
+        p_ac3dec->bsi.mixlevel = GetBits (&p_ac3dec->bit_stream,5);
 
         /* Get room type */
-        p_ac3dec->bsi.roomtyp = bitstream_get(&(p_ac3dec->bit_stream),2);
+        p_ac3dec->bsi.roomtyp = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->total_bits_read += 7;
     }
 
     /* If we're in dual mono mode then get some extra info */
     if (p_ac3dec->bsi.acmod == 0)
     {
         /* Get the dialogue normalization level two */
-        p_ac3dec->bsi.dialnorm2 = bitstream_get(&(p_ac3dec->bit_stream),5);
+        p_ac3dec->bsi.dialnorm2 = GetBits (&p_ac3dec->bit_stream,5);
 
         /* Does compression gain two exist? */
-        if ((p_ac3dec->bsi.compr2e = bitstream_get(&(p_ac3dec->bit_stream),1)))
+        if ((p_ac3dec->bsi.compr2e = GetBits (&p_ac3dec->bit_stream,1)))
         {
             /* Get compression gain two */
-            p_ac3dec->bsi.compr2 = bitstream_get(&(p_ac3dec->bit_stream),8);
+            p_ac3dec->bsi.compr2 = GetBits (&p_ac3dec->bit_stream,8);
+            p_ac3dec->total_bits_read += 8;
         }
 
         /* Does language code two exist? */
-        if ((p_ac3dec->bsi.langcod2e = bitstream_get(&(p_ac3dec->bit_stream),1)))
+        if ((p_ac3dec->bsi.langcod2e = GetBits (&p_ac3dec->bit_stream,1)))
         {
             /* Get langauge code two */
-            p_ac3dec->bsi.langcod2 = bitstream_get(&(p_ac3dec->bit_stream),8);
+            p_ac3dec->bsi.langcod2 = GetBits (&p_ac3dec->bit_stream,8);
+            p_ac3dec->total_bits_read += 8;
         }
 
         /* Does audio production info two exist? */
-        if ((p_ac3dec->bsi.audprodi2e = bitstream_get(&(p_ac3dec->bit_stream),1)))
+        if ((p_ac3dec->bsi.audprodi2e = GetBits (&p_ac3dec->bit_stream,1)))
         {
             /* Get mix level two */
-            p_ac3dec->bsi.mixlevel2 = bitstream_get(&(p_ac3dec->bit_stream),5);
+            p_ac3dec->bsi.mixlevel2 = GetBits (&p_ac3dec->bit_stream,5);
 
             /* Get room type two */
-            p_ac3dec->bsi.roomtyp2 = bitstream_get(&(p_ac3dec->bit_stream),2);
+            p_ac3dec->bsi.roomtyp2 = GetBits (&p_ac3dec->bit_stream,2);
+            p_ac3dec->total_bits_read += 7;
         }
+        p_ac3dec->total_bits_read += 8;
     }
 
     /* Get the copyright bit */
-    p_ac3dec->bsi.copyrightb = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->bsi.copyrightb = GetBits (&p_ac3dec->bit_stream,1);
 
     /* Get the original bit */
-    p_ac3dec->bsi.origbs = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->bsi.origbs = GetBits (&p_ac3dec->bit_stream,1);
 
     /* Does timecode one exist? */
-    if ((p_ac3dec->bsi.timecod1e = bitstream_get(&(p_ac3dec->bit_stream),1)))
+    if ((p_ac3dec->bsi.timecod1e = GetBits (&p_ac3dec->bit_stream,1)))
     {
-        p_ac3dec->bsi.timecod1 = bitstream_get(&(p_ac3dec->bit_stream),14);
+        p_ac3dec->bsi.timecod1 = GetBits (&p_ac3dec->bit_stream,14);
+        p_ac3dec->total_bits_read += 14;
     }
 
     /* Does timecode two exist? */
-    if ((p_ac3dec->bsi.timecod2e = bitstream_get(&(p_ac3dec->bit_stream),1)))
+    if ((p_ac3dec->bsi.timecod2e = GetBits (&p_ac3dec->bit_stream,1)))
     {
-        p_ac3dec->bsi.timecod2 = bitstream_get(&(p_ac3dec->bit_stream),14);
+        p_ac3dec->bsi.timecod2 = GetBits (&p_ac3dec->bit_stream,14);
+        p_ac3dec->total_bits_read += 14;
     }
 
     /* Does addition info exist? */
-    if ((p_ac3dec->bsi.addbsie = bitstream_get(&(p_ac3dec->bit_stream),1)))
+    if ((p_ac3dec->bsi.addbsie = GetBits (&p_ac3dec->bit_stream,1)))
     {
         u32 i;
 
         /* Get how much info is there */
-        p_ac3dec->bsi.addbsil = bitstream_get(&(p_ac3dec->bit_stream),6);
+        p_ac3dec->bsi.addbsil = GetBits (&p_ac3dec->bit_stream,6);
 
         /* Get the additional info */
         for (i=0;i<(p_ac3dec->bsi.addbsil + 1);i++)
         {
-            p_ac3dec->bsi.addbsi[i] = bitstream_get(&(p_ac3dec->bit_stream),8);
+            p_ac3dec->bsi.addbsi[i] = GetBits (&p_ac3dec->bit_stream,8);
         }
+        p_ac3dec->total_bits_read += 6 + 8 * (p_ac3dec->bsi.addbsil + 1);
     }
-
+    p_ac3dec->total_bits_read += 25;
+    
 #ifdef STATS
     parse_bsi_stats (p_ac3dec);
 #endif
@@ -280,35 +313,39 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     for (i=0; i < p_ac3dec->bsi.nfchans; i++)
     {
         /* Is this channel an interleaved 256 + 256 block ? */
-        p_ac3dec->audblk.blksw[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+        p_ac3dec->audblk.blksw[i] = GetBits (&p_ac3dec->bit_stream,1);
     }
 
     for (i=0; i < p_ac3dec->bsi.nfchans; i++)
     {
         /* Should we dither this channel? */
-        p_ac3dec->audblk.dithflag[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+        p_ac3dec->audblk.dithflag[i] = GetBits (&p_ac3dec->bit_stream,1);
     }
 
     /* Does dynamic range control exist? */
-    if ((p_ac3dec->audblk.dynrnge = bitstream_get(&(p_ac3dec->bit_stream),1)))
+    if ((p_ac3dec->audblk.dynrnge = GetBits (&p_ac3dec->bit_stream,1)))
     {
         /* Get dynamic range info */
-        p_ac3dec->audblk.dynrng = bitstream_get(&(p_ac3dec->bit_stream),8);
+        p_ac3dec->audblk.dynrng = GetBits (&p_ac3dec->bit_stream,8);
+        p_ac3dec->total_bits_read += 8;
     }
 
     /* If we're in dual mono mode then get the second channel DR info */
     if (p_ac3dec->bsi.acmod == 0)
     {
         /* Does dynamic range control two exist? */
-        if ((p_ac3dec->audblk.dynrng2e = bitstream_get(&(p_ac3dec->bit_stream),1)))
+        if ((p_ac3dec->audblk.dynrng2e = GetBits (&p_ac3dec->bit_stream,1)))
         {
             /* Get dynamic range info */
-            p_ac3dec->audblk.dynrng2 = bitstream_get(&(p_ac3dec->bit_stream),8);
+            p_ac3dec->audblk.dynrng2 = GetBits (&p_ac3dec->bit_stream,8);
+            p_ac3dec->total_bits_read += 8;
         }
+        p_ac3dec->total_bits_read += 1;
     }
 
     /* Does coupling strategy exist? */
-    p_ac3dec->audblk.cplstre = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->audblk.cplstre = GetBits (&p_ac3dec->bit_stream,1);
+    p_ac3dec->total_bits_read += 2 + 2 * p_ac3dec->bsi.nfchans;
 
     if ((!blknum) && (!p_ac3dec->audblk.cplstre))
     {
@@ -318,19 +355,21 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     if (p_ac3dec->audblk.cplstre)
     {
         /* Is coupling turned on? */
-        if ((p_ac3dec->audblk.cplinu = bitstream_get(&(p_ac3dec->bit_stream),1)))
+        if ((p_ac3dec->audblk.cplinu = GetBits (&p_ac3dec->bit_stream,1)))
         {
             int nb_coupled_channels;
 
             nb_coupled_channels = 0;
             for (i=0; i < p_ac3dec->bsi.nfchans; i++)
             {
-                p_ac3dec->audblk.chincpl[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+                p_ac3dec->audblk.chincpl[i] = GetBits (&p_ac3dec->bit_stream,1);
                 if (p_ac3dec->audblk.chincpl[i])
                 {
                     nb_coupled_channels++;
                 }
             }
+            p_ac3dec->total_bits_read += p_ac3dec->bsi.nfchans;
+            
             if (nb_coupled_channels < 2)
             {
                 return 1;
@@ -338,10 +377,12 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
             if (p_ac3dec->bsi.acmod == 0x2)
             {
-                p_ac3dec->audblk.phsflginu = bitstream_get(&(p_ac3dec->bit_stream),1);
+                p_ac3dec->audblk.phsflginu = GetBits (&p_ac3dec->bit_stream,1);
+                p_ac3dec->total_bits_read += 1;
             }
-            p_ac3dec->audblk.cplbegf = bitstream_get(&(p_ac3dec->bit_stream),4);
-            p_ac3dec->audblk.cplendf = bitstream_get(&(p_ac3dec->bit_stream),4);
+            p_ac3dec->audblk.cplbegf = GetBits (&p_ac3dec->bit_stream,4);
+            p_ac3dec->audblk.cplendf = GetBits (&p_ac3dec->bit_stream,4);
+            p_ac3dec->total_bits_read += 8;
 
             if (p_ac3dec->audblk.cplbegf > p_ac3dec->audblk.cplendf + 2)
             {
@@ -360,10 +401,12 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
             for (i=1; i< p_ac3dec->audblk.ncplsubnd; i++)
             {
-                p_ac3dec->audblk.cplbndstrc[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+                p_ac3dec->audblk.cplbndstrc[i] = GetBits (&p_ac3dec->bit_stream,1);
                 p_ac3dec->audblk.ncplbnd -= p_ac3dec->audblk.cplbndstrc[i];
             }
+            p_ac3dec->total_bits_read += p_ac3dec->audblk.ncplsubnd - 1;
         }
+        p_ac3dec->total_bits_read += 1;
     }
 
     if (p_ac3dec->audblk.cplinu)
@@ -377,7 +420,7 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
             }
 
             /* Is there new coupling co-ordinate info? */
-            p_ac3dec->audblk.cplcoe[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+            p_ac3dec->audblk.cplcoe[i] = GetBits (&p_ac3dec->bit_stream,1);
 
             if ((!blknum) && (!p_ac3dec->audblk.cplcoe[i]))
             {
@@ -386,14 +429,17 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
             if (p_ac3dec->audblk.cplcoe[i])
             {
-                p_ac3dec->audblk.mstrcplco[i] = bitstream_get(&(p_ac3dec->bit_stream),2);
+                p_ac3dec->audblk.mstrcplco[i] = GetBits (&p_ac3dec->bit_stream,2);
+                p_ac3dec->total_bits_read += 2;
                 for (j=0;j < p_ac3dec->audblk.ncplbnd; j++)
                 {
-                    p_ac3dec->audblk.cplcoexp[i][j] = bitstream_get(&(p_ac3dec->bit_stream),4);
-                    p_ac3dec->audblk.cplcomant[i][j] = bitstream_get(&(p_ac3dec->bit_stream),4);
+                    p_ac3dec->audblk.cplcoexp[i][j] = GetBits (&p_ac3dec->bit_stream,4);
+                    p_ac3dec->audblk.cplcomant[i][j] = GetBits (&p_ac3dec->bit_stream,4);
                 }
+                p_ac3dec->total_bits_read += 8 * p_ac3dec->audblk.ncplbnd;
             }
         }
+        p_ac3dec->total_bits_read += p_ac3dec->bsi.nfchans;
 
         /* If we're in dual mono mode, there's going to be some phase info */
         if ((p_ac3dec->bsi.acmod == 0x2) && p_ac3dec->audblk.phsflginu &&
@@ -401,8 +447,9 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
         {
             for (j=0; j < p_ac3dec->audblk.ncplbnd; j++)
             {
-                p_ac3dec->audblk.phsflg[j] = bitstream_get(&(p_ac3dec->bit_stream),1);
+                p_ac3dec->audblk.phsflg[j] = GetBits (&p_ac3dec->bit_stream,1);
             }
+            p_ac3dec->total_bits_read += p_ac3dec->audblk.ncplbnd;
 
         }
     }
@@ -410,7 +457,8 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     /* If we're in dual mono mode, there may be a rematrix strategy */
     if (p_ac3dec->bsi.acmod == 0x2)
     {
-        p_ac3dec->audblk.rematstr = bitstream_get(&(p_ac3dec->bit_stream),1);
+        p_ac3dec->audblk.rematstr = GetBits (&p_ac3dec->bit_stream,1);
+        p_ac3dec->total_bits_read += 1;
 
         if ((!blknum) && (!p_ac3dec->audblk.rematstr))
         {
@@ -423,29 +471,33 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
             {
                 for (i = 0; i < 4; i++)
                 {
-                    p_ac3dec->audblk.rematflg[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+                    p_ac3dec->audblk.rematflg[i] = GetBits (&p_ac3dec->bit_stream,1);
                 }
+                p_ac3dec->total_bits_read += 4;
             }
             if ((p_ac3dec->audblk.cplbegf > 2) && p_ac3dec->audblk.cplinu)
             {
                 for (i = 0; i < 4; i++)
                 {
-                    p_ac3dec->audblk.rematflg[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+                    p_ac3dec->audblk.rematflg[i] = GetBits (&p_ac3dec->bit_stream,1);
                 }
+                p_ac3dec->total_bits_read += 4;
             }
             if ((p_ac3dec->audblk.cplbegf <= 2) && p_ac3dec->audblk.cplinu)
             {
                 for (i = 0; i < 3; i++)
                 {
-                    p_ac3dec->audblk.rematflg[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+                    p_ac3dec->audblk.rematflg[i] = GetBits (&p_ac3dec->bit_stream,1);
                 }
+                p_ac3dec->total_bits_read += 3;
             }
             if ((p_ac3dec->audblk.cplbegf == 0) && p_ac3dec->audblk.cplinu)
             {
                 for (i = 0; i < 2; i++)
                 {
-                    p_ac3dec->audblk.rematflg[i] = bitstream_get(&(p_ac3dec->bit_stream),1);
+                    p_ac3dec->audblk.rematflg[i] = GetBits (&p_ac3dec->bit_stream,1);
                 }
+                p_ac3dec->total_bits_read += 2;
             }
         }
     }
@@ -453,7 +505,8 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     if (p_ac3dec->audblk.cplinu)
     {
         /* Get the coupling channel exponent strategy */
-        p_ac3dec->audblk.cplexpstr = bitstream_get(&(p_ac3dec->bit_stream),2);
+        p_ac3dec->audblk.cplexpstr = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->total_bits_read += 2;
 
         if ((!blknum) && (p_ac3dec->audblk.cplexpstr == EXP_REUSE))
         {
@@ -474,7 +527,8 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
     for (i = 0; i < p_ac3dec->bsi.nfchans; i++)
     {
-        p_ac3dec->audblk.chexpstr[i] = bitstream_get(&(p_ac3dec->bit_stream),2);
+        p_ac3dec->audblk.chexpstr[i] = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->total_bits_read += 2;
 
         if ((!blknum) && (p_ac3dec->audblk.chexpstr[i] == EXP_REUSE))
         {
@@ -485,7 +539,8 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     /* Get the exponent strategy for lfe channel */
     if (p_ac3dec->bsi.lfeon)
     {
-        p_ac3dec->audblk.lfeexpstr = bitstream_get(&(p_ac3dec->bit_stream),1);
+        p_ac3dec->audblk.lfeexpstr = GetBits (&p_ac3dec->bit_stream,1);
+        p_ac3dec->total_bits_read += 1;
 
         if ((!blknum) && (p_ac3dec->audblk.lfeexpstr == EXP_REUSE))
         {
@@ -506,7 +561,8 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
             }
             else
             {
-                p_ac3dec->audblk.chbwcod[i] = bitstream_get(&(p_ac3dec->bit_stream),6);
+                p_ac3dec->audblk.chbwcod[i] = GetBits (&p_ac3dec->bit_stream,6);
+                p_ac3dec->total_bits_read += 6;
 
                 if (p_ac3dec->audblk.chbwcod[i] > 60)
                 {
@@ -525,10 +581,12 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     /* Get the coupling exponents if they exist */
     if (p_ac3dec->audblk.cplinu && (p_ac3dec->audblk.cplexpstr != EXP_REUSE))
     {
-        p_ac3dec->audblk.cplabsexp = bitstream_get(&(p_ac3dec->bit_stream),4);
+        p_ac3dec->audblk.cplabsexp = GetBits (&p_ac3dec->bit_stream,4);
+        p_ac3dec->total_bits_read += 4;
         for (i=0; i< p_ac3dec->audblk.ncplgrps;i++)
         {
-            p_ac3dec->audblk.cplexps[i] = bitstream_get(&(p_ac3dec->bit_stream),7);
+            p_ac3dec->audblk.cplexps[i] = GetBits (&p_ac3dec->bit_stream,7);
+            p_ac3dec->total_bits_read += 7;
 
             if (p_ac3dec->audblk.cplexps[i] >= 125)
             {
@@ -542,29 +600,34 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     {
         if (p_ac3dec->audblk.chexpstr[i] != EXP_REUSE)
         {
-            p_ac3dec->audblk.exps[i][0] = bitstream_get(&(p_ac3dec->bit_stream),4);
+            p_ac3dec->audblk.exps[i][0] = GetBits (&p_ac3dec->bit_stream,4);
+            p_ac3dec->total_bits_read += 4;
             for (j=1; j<=p_ac3dec->audblk.nchgrps[i];j++)
             {
-                p_ac3dec->audblk.exps[i][j] = bitstream_get(&(p_ac3dec->bit_stream),7);
+                p_ac3dec->audblk.exps[i][j] = GetBits (&p_ac3dec->bit_stream,7);
+                p_ac3dec->total_bits_read += 7;
                 if (p_ac3dec->audblk.exps[i][j] >= 125)
                 {
                     return 1;
                 }
             }
-            p_ac3dec->audblk.gainrng[i] = bitstream_get(&(p_ac3dec->bit_stream),2);
+            p_ac3dec->audblk.gainrng[i] = GetBits (&p_ac3dec->bit_stream,2);
+            p_ac3dec->total_bits_read += 2;
         }
     }
 
     /* Get the lfe channel exponents */
     if (p_ac3dec->bsi.lfeon && (p_ac3dec->audblk.lfeexpstr != EXP_REUSE))
     {
-        p_ac3dec->audblk.lfeexps[0] = bitstream_get(&(p_ac3dec->bit_stream),4);
-        p_ac3dec->audblk.lfeexps[1] = bitstream_get(&(p_ac3dec->bit_stream),7);
+        p_ac3dec->audblk.lfeexps[0] = GetBits (&p_ac3dec->bit_stream,4);
+        p_ac3dec->audblk.lfeexps[1] = GetBits (&p_ac3dec->bit_stream,7);
+        p_ac3dec->total_bits_read += 11;
         if (p_ac3dec->audblk.lfeexps[1] >= 125)
         {
             return 1;
         }
-        p_ac3dec->audblk.lfeexps[2] = bitstream_get(&(p_ac3dec->bit_stream),7);
+        p_ac3dec->audblk.lfeexps[2] = GetBits (&p_ac3dec->bit_stream,7);
+        p_ac3dec->total_bits_read += 7;
         if (p_ac3dec->audblk.lfeexps[2] >= 125)
         {
             return 1;
@@ -572,7 +635,8 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
     }
 
     /* Get the parametric bit allocation parameters */
-    p_ac3dec->audblk.baie = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->audblk.baie = GetBits (&p_ac3dec->bit_stream,1);
+    p_ac3dec->total_bits_read += 1;
 
     if ((!blknum) && (!p_ac3dec->audblk.baie))
     {
@@ -581,15 +645,16 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
     if (p_ac3dec->audblk.baie)
     {
-        p_ac3dec->audblk.sdcycod = bitstream_get(&(p_ac3dec->bit_stream),2);
-        p_ac3dec->audblk.fdcycod = bitstream_get(&(p_ac3dec->bit_stream),2);
-        p_ac3dec->audblk.sgaincod = bitstream_get(&(p_ac3dec->bit_stream),2);
-        p_ac3dec->audblk.dbpbcod = bitstream_get(&(p_ac3dec->bit_stream),2);
-        p_ac3dec->audblk.floorcod = bitstream_get(&(p_ac3dec->bit_stream),3);
+        p_ac3dec->audblk.sdcycod = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->audblk.fdcycod = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->audblk.sgaincod = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->audblk.dbpbcod = GetBits (&p_ac3dec->bit_stream,2);
+        p_ac3dec->audblk.floorcod = GetBits (&p_ac3dec->bit_stream,3);
+        p_ac3dec->total_bits_read += 11;
     }
 
     /* Get the SNR off set info if it exists */
-    p_ac3dec->audblk.snroffste = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->audblk.snroffste = GetBits (&p_ac3dec->bit_stream,1);
     if ((!blknum) && (!p_ac3dec->audblk.snroffste))
     {
         return 1;
@@ -597,30 +662,35 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
     if (p_ac3dec->audblk.snroffste)
     {
-        p_ac3dec->audblk.csnroffst = bitstream_get(&(p_ac3dec->bit_stream),6);
+        p_ac3dec->audblk.csnroffst = GetBits (&p_ac3dec->bit_stream,6);
+        p_ac3dec->total_bits_read += 6;
 
         if (p_ac3dec->audblk.cplinu)
         {
-            p_ac3dec->audblk.cplfsnroffst = bitstream_get(&(p_ac3dec->bit_stream),4);
-            p_ac3dec->audblk.cplfgaincod = bitstream_get(&(p_ac3dec->bit_stream),3);
+            p_ac3dec->audblk.cplfsnroffst = GetBits (&p_ac3dec->bit_stream,4);
+            p_ac3dec->audblk.cplfgaincod = GetBits (&p_ac3dec->bit_stream,3);
+            p_ac3dec->total_bits_read += 7;
         }
 
         for (i = 0;i < p_ac3dec->bsi.nfchans; i++)
         {
-            p_ac3dec->audblk.fsnroffst[i] = bitstream_get(&(p_ac3dec->bit_stream),4);
-            p_ac3dec->audblk.fgaincod[i] = bitstream_get(&(p_ac3dec->bit_stream),3);
+            p_ac3dec->audblk.fsnroffst[i] = GetBits (&p_ac3dec->bit_stream,4);
+            p_ac3dec->audblk.fgaincod[i] = GetBits (&p_ac3dec->bit_stream,3);
         }
+        p_ac3dec->total_bits_read += 7 * p_ac3dec->bsi.nfchans;
         if (p_ac3dec->bsi.lfeon)
         {
-            p_ac3dec->audblk.lfefsnroffst = bitstream_get(&(p_ac3dec->bit_stream),4);
-            p_ac3dec->audblk.lfefgaincod = bitstream_get(&(p_ac3dec->bit_stream),3);
+            p_ac3dec->audblk.lfefsnroffst = GetBits (&p_ac3dec->bit_stream,4);
+            p_ac3dec->audblk.lfefgaincod = GetBits (&p_ac3dec->bit_stream,3);
+            p_ac3dec->total_bits_read += 7;
         }
     }
 
     /* Get coupling leakage info if it exists */
     if (p_ac3dec->audblk.cplinu)
     {
-        p_ac3dec->audblk.cplleake = bitstream_get(&(p_ac3dec->bit_stream),1);
+        p_ac3dec->audblk.cplleake = GetBits (&p_ac3dec->bit_stream,1);
+        p_ac3dec->total_bits_read += 1;
         if ((!blknum) && (!p_ac3dec->audblk.cplleake))
         {
             return 1;
@@ -628,19 +698,22 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
         if (p_ac3dec->audblk.cplleake)
         {
-            p_ac3dec->audblk.cplfleak = bitstream_get(&(p_ac3dec->bit_stream),3);
-            p_ac3dec->audblk.cplsleak = bitstream_get(&(p_ac3dec->bit_stream),3);
+            p_ac3dec->audblk.cplfleak = GetBits (&p_ac3dec->bit_stream,3);
+            p_ac3dec->audblk.cplsleak = GetBits (&p_ac3dec->bit_stream,3);
+            p_ac3dec->total_bits_read += 6;
         }
     }
 
     /* Get the delta bit alloaction info */
-    p_ac3dec->audblk.deltbaie = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->audblk.deltbaie = GetBits (&p_ac3dec->bit_stream,1);
+    p_ac3dec->total_bits_read += 1;
 
     if (p_ac3dec->audblk.deltbaie)
     {
         if (p_ac3dec->audblk.cplinu)
         {
-            p_ac3dec->audblk.cpldeltbae = bitstream_get(&(p_ac3dec->bit_stream),2);
+            p_ac3dec->audblk.cpldeltbae = GetBits (&p_ac3dec->bit_stream,2);
+            p_ac3dec->total_bits_read += 2;
             if (p_ac3dec->audblk.cpldeltbae == 3)
             {
                 return 1;
@@ -649,7 +722,8 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
         for (i = 0;i < p_ac3dec->bsi.nfchans; i++)
         {
-            p_ac3dec->audblk.deltbae[i] = bitstream_get(&(p_ac3dec->bit_stream),2);
+            p_ac3dec->audblk.deltbae[i] = GetBits (&p_ac3dec->bit_stream,2);
+            p_ac3dec->total_bits_read += 2;
             if (p_ac3dec->audblk.deltbae[i] == 3)
             {
                 return 1;
@@ -658,45 +732,49 @@ int parse_audblk (ac3dec_t * p_ac3dec, int blknum)
 
         if (p_ac3dec->audblk.cplinu && (p_ac3dec->audblk.cpldeltbae == DELTA_BIT_NEW))
         {
-            p_ac3dec->audblk.cpldeltnseg = bitstream_get(&(p_ac3dec->bit_stream),3);
+            p_ac3dec->audblk.cpldeltnseg = GetBits (&p_ac3dec->bit_stream,3);
             for (i = 0;i < p_ac3dec->audblk.cpldeltnseg + 1; i++)
             {
-                p_ac3dec->audblk.cpldeltoffst[i] = bitstream_get(&(p_ac3dec->bit_stream),5);
-                p_ac3dec->audblk.cpldeltlen[i] = bitstream_get(&(p_ac3dec->bit_stream),4);
-                p_ac3dec->audblk.cpldeltba[i] = bitstream_get(&(p_ac3dec->bit_stream),3);
+                p_ac3dec->audblk.cpldeltoffst[i] = GetBits (&p_ac3dec->bit_stream,5);
+                p_ac3dec->audblk.cpldeltlen[i] = GetBits (&p_ac3dec->bit_stream,4);
+                p_ac3dec->audblk.cpldeltba[i] = GetBits (&p_ac3dec->bit_stream,3);
             }
+            p_ac3dec->total_bits_read += 12 * (p_ac3dec->audblk.cpldeltnseg + 1) + 3;
         }
 
         for (i = 0; i < p_ac3dec->bsi.nfchans; i++)
         {
             if (p_ac3dec->audblk.deltbae[i] == DELTA_BIT_NEW)
             {
-                p_ac3dec->audblk.deltnseg[i] = bitstream_get(&(p_ac3dec->bit_stream),3);
+                p_ac3dec->audblk.deltnseg[i] = GetBits (&p_ac3dec->bit_stream,3);
 //                if (p_ac3dec->audblk.deltnseg[i] >= 8)
 //                    fprintf (stderr, "parse debug: p_ac3dec->audblk.deltnseg[%i] == %i\n", i, p_ac3dec->audblk.deltnseg[i]);
                 for (j = 0; j < p_ac3dec->audblk.deltnseg[i] + 1; j++)
                 {
-                    p_ac3dec->audblk.deltoffst[i][j] = bitstream_get(&(p_ac3dec->bit_stream),5);
-                    p_ac3dec->audblk.deltlen[i][j] = bitstream_get(&(p_ac3dec->bit_stream),4);
-                    p_ac3dec->audblk.deltba[i][j] = bitstream_get(&(p_ac3dec->bit_stream),3);
+                    p_ac3dec->audblk.deltoffst[i][j] = GetBits (&p_ac3dec->bit_stream,5);
+                    p_ac3dec->audblk.deltlen[i][j] = GetBits (&p_ac3dec->bit_stream,4);
+                    p_ac3dec->audblk.deltba[i][j] = GetBits (&p_ac3dec->bit_stream,3);
                 }
+                p_ac3dec->total_bits_read += 12 * (p_ac3dec->audblk.deltnseg[i] + 1) + 3;
             }
         }
     }
 
     /* Check to see if there's any dummy info to get */
-    p_ac3dec->audblk.skiple = bitstream_get(&(p_ac3dec->bit_stream),1);
+    p_ac3dec->audblk.skiple = GetBits (&p_ac3dec->bit_stream,1);
+    p_ac3dec->total_bits_read += 1;
 
     if (p_ac3dec->audblk.skiple)
     {
-        p_ac3dec->audblk.skipl = bitstream_get(&(p_ac3dec->bit_stream),9);
+        p_ac3dec->audblk.skipl = GetBits (&p_ac3dec->bit_stream,9);
 
         for (i = 0; i < p_ac3dec->audblk.skipl ; i++)
         {
-            bitstream_get(&(p_ac3dec->bit_stream),8);
+            GetBits (&p_ac3dec->bit_stream,8);
         }
+        p_ac3dec->total_bits_read += 8 * p_ac3dec->audblk.skipl + 9;
     }
-
+    
 #ifdef STATS
 //    parse_audblk_stats(p_ac3dec);
 #endif
@@ -709,21 +787,21 @@ void parse_auxdata (ac3dec_t * p_ac3dec)
     int i;
     int skip_length;
 
-    skip_length = (p_ac3dec->syncinfo.frame_size * 16) - p_ac3dec->bit_stream.total_bits_read - 17 - 1;
+    skip_length = (p_ac3dec->syncinfo.frame_size * 16) - p_ac3dec->total_bits_read - 17 - 1;
 
     for (i = 0; i < skip_length; i++)
     {
-        bitstream_get(&(p_ac3dec->bit_stream),1);
+        RemoveBits (&p_ac3dec->bit_stream,1);
     }
 
     /* get the auxdata exists bit */
-    bitstream_get(&(p_ac3dec->bit_stream),1);
+    RemoveBits (&p_ac3dec->bit_stream,1);
     
     /* Skip the CRC reserved bit */
-    bitstream_get(&(p_ac3dec->bit_stream),1);
+    RemoveBits (&p_ac3dec->bit_stream,1);
 
     /* Get the crc */
-    bitstream_get(&(p_ac3dec->bit_stream),16);
+    RemoveBits (&p_ac3dec->bit_stream,16);
 }
 
 void parse_bsi_stats (ac3dec_t * p_ac3dec) /*Some stats */
@@ -751,7 +829,7 @@ void parse_bsi_stats (ac3dec_t * p_ac3dec) /*Some stats */
     };
  */
     
-    static int  i;
+    static int  i=0;
     
     if ( !i )
     {
