@@ -2,7 +2,7 @@
  * vpar_synchro.c : frame dropping routines
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: vpar_synchro.c,v 1.85 2001/02/12 18:18:18 massiot Exp $
+ * $Id: vpar_synchro.c,v 1.88 2001/04/27 19:29:11 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Samuel Hocevar <sam@via.ecp.fr>
@@ -96,6 +96,8 @@
  *****************************************************************************/
 #include "defs.h"
 
+#include <string.h>                                    /* memcpy(), memset() */
+
 #include "config.h"
 #include "common.h"
 #include "threads.h"
@@ -222,7 +224,7 @@ boolean_t vpar_SynchroChoose( vpar_thread_t * p_vpar, int i_coding_type,
         mtime_t         now, period, tau_yuv;
         mtime_t         pts = 0;
         boolean_t       b_decode = 0;
-#ifdef DEBUG_VPAR
+#ifdef TRACE_VPAR
         char            p_date[MSTRTIME_MAX_SIZE];
 #endif
 
@@ -319,7 +321,7 @@ boolean_t vpar_SynchroChoose( vpar_thread_t * p_vpar, int i_coding_type,
 #ifdef VDEC_SMP
         vlc_mutex_unlock( &p_vpar->synchro.fifo_lock );
 #endif
-#ifdef DEBUG_VPAR
+#ifdef TRACE_VPAR
         intf_DbgMsg("vpar synchro debug: %s picture scheduled for %s, %s (%lld)",
                     i_coding_type == B_CODING_TYPE ? "B" :
                     (i_coding_type == P_CODING_TYPE ? "P" : "I"),
@@ -370,7 +372,8 @@ void vpar_SynchroDecode( vpar_thread_t * p_vpar, int i_coding_type,
     else
     {
         /* FIFO full, panic() */
-        intf_ErrMsg("vpar error: synchro fifo full, estimations will be biased");
+        intf_ErrMsg("vpar error: synchro fifo full, estimations will be biased (%d:%d)",
+                    p_vpar->synchro.i_start, p_vpar->synchro.i_end);
     }
 #ifdef VDEC_SMP
     vlc_mutex_unlock( &p_vpar->synchro.fifo_lock );
@@ -389,10 +392,11 @@ void vpar_SynchroEnd( vpar_thread_t * p_vpar, int i_garbage )
     vlc_mutex_lock( &p_vpar->synchro.fifo_lock );
 #endif
 
+    i_coding_type = p_vpar->synchro.pi_coding_types[p_vpar->synchro.i_start];
+
     if( !i_garbage )
     {
         tau = mdate() - p_vpar->synchro.p_date_fifo[p_vpar->synchro.i_start];
-        i_coding_type = p_vpar->synchro.pi_coding_types[p_vpar->synchro.i_start];
 
         /* If duration too high, something happened (pause ?), so don't
          * take it into account. */
@@ -410,12 +414,18 @@ void vpar_SynchroEnd( vpar_thread_t * p_vpar, int i_garbage )
             }
         }
 
-#ifdef DEBUG_VPAR
+#ifdef TRACE_VPAR
         intf_DbgMsg("vpar synchro debug: finished decoding %s (%lld)",
                     i_coding_type == B_CODING_TYPE ? "B" :
                     (i_coding_type == P_CODING_TYPE ? "P" : "I"), tau);
 #endif
     }
+    else
+    {
+        intf_DbgMsg("vpar synchro debug: aborting %s",
+                    i_coding_type == B_CODING_TYPE ? "B" :
+                    (i_coding_type == P_CODING_TYPE ? "P" : "I"));
+    }
 
     FIFO_INCREMENT( i_start );
 
